#!/usr/bin/env python
#coding=utf-8

import uuid
from sqlalchemy import *
from sqlalchemy.orm import mapper, sessionmaker,relation
from datetime import  datetime,date
import re
import string
from pinyin import *
def uid():
    id=uuid.uuid1().hex
    return id
class DB(object):
    def __init__ (self,dbfile):
        self.db = create_engine(dbfile, echo = False)
        self.create_db()
        Session = sessionmaker(bind = self.db, autoflush = True, transactional = True)
        self.session = Session()
        mapper(Data,self.table_datas,properties={"link":relation(Data,self.table_datas_tags)})
        mapper(Tag,self.table_tags)
        mapper(Datas_tags,self.table_datas_tags)
    def get_data (self,data_id):
        data=self.session.query(Data).get(data_id)
        if data==None:
            data=Data(0,"")
        else:
            data.tags=data.get_tags(self)
        return data
    def del_data(self,data_id):
        data=self.session.query(Data).get(data_id)
        if not data==None:
            data.clear_tags(self)
            self.session.delete(data)
            self.session.commit()
    def link_tag (self,data_tag):
        data_tag.updated_at=datetime.now()
        self.session.save(data_tag)
        value=data_tag.value
        tag=self.session.query(Tag).get(data_tag.tag)
        if tag==None :
            tag=Tag(data_tag.tag,0,0,)
        tag.count=tag.count+1
        tag.sum=tag.sum+value
        self.session.save_or_update(tag)
        self.session.commit()
    def remove_link(self,data_tag):
        tag=self.session.query(Tag).get(data_tag.tag)
        value=data_tag.value
        tag.sum=tag.sum-value
        tag.count=tag.count-1
        self.session.delete(data_tag)
        if tag.count==0:
            self.session.delete(tag)
        else:
            self.session.save_or_update(tag)
        self.session.commit()
    def get_tag(self,data):
            return self.session.query(Datas_tags).filter(Datas_tags.data_id==data.id).all()
    def query_tags(self,tags):
        Q=self.session.query(Data)
        re1=re.compile("([\d]{4,4}-[\d]{2,2}-[\d]{2,2}){0,1}~([\d]{4,4}-[\d]{2,2}-[\d]{2,2}){0,1}")
        re2=re1.search(tags)
        if re2!=None:
            if re2.groups()[0] !=None:
                Q=Q.filter(Data.date>=re2.groups()[0])
            if re2.groups()[1] !=None:
                Q=Q.filter(Data.date<=re2.groups()[1])
            return Q
            tags=tags[:re2.span()[0]]+tags[re2.span()[1]:]
        tmp=Data(0,"")
        tag=ConvertTags(tags,tmp)        
        for item in tag:
            if item.tag!="":
                if item.reversed:
                    Q=Q.filter(~Data.link.any(Datas_tags.tag==item.tag))
                else:
                    Q=Q.filter(Data.link.any(Datas_tags.tag==item.tag))
        return Q
    def create_db (self):
        metadata = MetaData()
        self.table_datas = Table('datas', metadata,
                            Column('id', String(32), primary_key = True),
                            Column('value', Integer),
                            Column('comment', Text),
                            Column('updated_at', DateTime),
                            Column("date",Date)
                            )
        self.table_tags = Table('tags', metadata,
                            Column('name', Unicode(32),primary_key = True),
                            Column('count', Integer),
                            Column('sum', Integer),
                            Column('shortcut',String(32))
                                                        )
        self.table_datas_tags = Table('datas_tags', metadata,
                                      Column('id', String(32), primary_key = True),
                                      Column('data_id', String(32),ForeignKey("datas.id")),
                                      Column('value', Integer),
                                      Column('tag', Unicode(32)),
                                      Column("reversed",Boolean),
                                      Column('updated_at', DateTime))
        self.table_db_info = Table('JQMdbinfo', metadata,
                                      Column('id', String(32), primary_key = True),
                                      Column('version', String(32), ForeignKey('datas.id'))
                                    )
        self.table_db_sync = Table('sync_info', metadata,
                                      Column('id', String(32), primary_key = True),
                                      Column('updated_at', DateTime)
                                                                          )
        metadata.create_all(self.db)
    def sync(self):
        pass
        
class Data(object):
    def __init__ (self,value,comment="",tags="",ddate="",id = ""):
        if (id==""):
            id=uid()
        if (ddate==""):
            ddate=date.today()
        self.id=id
        self.value=value
        self.comment=comment
        self.tags=tags
        self.date=ddate
    def save(self,db=None):
        if db==None:
            db=DBFile   
        self.updated_at=datetime.now()
        db.session.save_or_update(self)
        db.session.commit()
        self.set_tags(self.tags)
    def clear_tags (self,db=None):
        if db==None:
            db=DBFile
        clear_tag=db.session.query(Datas_tags).filter(Datas_tags.data_id==self.id).all()
        for item in clear_tag:
            db.remove_link(item)
    def set_tags(self,tags,db=None):
        if db==None:
            db=DBFile
        self.clear_tags(db)
        alltag=ConvertTags(tags,self)
        for item in alltag:
            db.link_tag(item)
    def get_tags(self,db=None):
        if db==None:
            db=DBFile
        tags=db.get_tag(self)
        tagstext=""
        for item in tags:
            tag=item.tag
            if item.reversed:
                tag="-"+tag
            if tag.find(" ")>0:
                tag='"'+tag+'"'
            tagstext=tagstext+" "+tag
        if tagstext != "":
            tagstext=tagstext[1:]
        return tagstext        
    

class Datas_tags(object):
    def __init__ (self,data,tag,reversed,id=""):
        if (id==""):
            id=uid()
        self.id=id
        self.data_id=data.id
        self.value=data.value
        self.tag=tag
        self.reversed=reversed
class Tag(object):
    def __init__ (self,name,count=0,sum=0,shortcut=""):
        self.name=name
        self.count=count
        self.sum=sum
        if shortcut=="":
            for x in name:
                if x==" ":continue
                C=get_cn_first_letter(x,"unicode")
                if C=="":
                    self.shortcut=""
                    return
                shortcut=shortcut+C
        self.shortcut=shortcut
def ConvertTags(tags,data):
    links=[]
    if tags.strip()=="":tags=unicode("现金")
    tags=" "+tags+ " "
    re1=re.compile(u" ([\'\"“”‘’]\\S.*\\S[\'\"“”‘’]) ",)
    t=re1.split(tags)
    for item in t:
        if item=="":continue
        if u"[\'\"“”‘’]".find(item[0])>0:
            item=item[1:(len(item)-1)]
            data_tag=Datas_tags(data,item,False)
            if (item[0]==u"-")or(item[0]==u"－"):
                data_tag.tag=item[1:]
                data_tag.reversed=True
            data_tag.value=data.value
            if data_tag.reversed==True :
                data_tag.value=-data_tag.value
            links.append(data_tag)
        else:
            t2=item.split(" ")
            for item1 in t2:
                if item1=="":continue
                data_tag=Datas_tags(data,item1,False)                
                if (item1[0]==u"-")or(item1[0]==u"－"):
                    data_tag.tag=item1[1:]
                    data_tag.reversed=True        
                data_tag.value=data.value
                if data_tag.reversed==True :
                    data_tag.value=-data_tag.value
                links.append(data_tag)
    return links

    

class syncdate(object):
    def __init__ (self,name,updated_at):
        self.name=name
        self.updated_at=updated_at
            

DBFile=DB('sqlite:///data.db')


if __name__ == '__main__':
##    data=Data(1,"test","a")
##    data.save()
##    data=Data(2,"test","a b")
##    data.save()
##    data=Data(3,"test","a b c")
##    data.save()
##    data=Data(4,"test","a b c d")
##    data.save()##    data=Data(5,"test","a b c d e")
##    data.save()
##    data=Data(6,"test","c d e f g")
##    data.save()
##    data=Data(7,"test","d e f g h")
##    data.save()
##    data=Data(8,"test","e f g h i")
##    data.save()

##   DBFile.session.query()
##   data.save
##    DBFile.del_data("83414238454c11ddbb5a001636b2ab9b")
    tmp=DBFile.session.query(Data).first().date
    print type(tmp)
